@@ -9,9 +9,9 @@ from covsirphy.util.validator import Validator
 from covsirphy.util.term import Term
 from covsirphy.gis.gis import GIS
 from covsirphy.downloading.downloader import DataDownloader
-from covsirphy.engineering.cleaner import _DataCleaner
-from covsirphy.engineering.transformer import _DataTransformer
-from covsirphy.engineering.complement import _ComplementHandler
+from covsirphy.engineering._cleaner import _DataCleaner
+from covsirphy.engineering._transformer import _DataTransformer
+from covsirphy.engineering._complement import _ComplementHandler
 
 
 class DataEngineer(Term):
@@ -140,8 +140,10 @@ class DataEngineer(Term):
 
         Note:
             For "convert_date", keyword arguments of pandas.to_datetime() including "dayfirst (bool): whether date format is DD/MM or not" can be used.
+
+        Note:
+            For "resample", `date_range=<tuple of (str or None, str or None) or None>)` can be applied as keyword arguments to set the range.
         """
-        citations = self._gis.citations(variables=None)
         cleaner = _DataCleaner(data=self._gis.all(), layers=self._layers, date=self.DATE)
         kind_dict = {
             "convert_date": cleaner.convert_date,
@@ -152,10 +154,7 @@ class DataEngineer(Term):
         selected = Validator(kinds, "kind").sequence(default=all_kinds, candidates=all_kinds)
         for kind in selected:
             kind_dict[kind](**Validator(kwargs, "keyword arguments").kwargs(functions=kind_dict[kind], default=None))
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=cleaner.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(cleaner)
 
     def transform(self):
         """Transform all registered data, calculating the number of susceptible and infected cases.
@@ -170,14 +169,10 @@ class DataEngineer(Term):
             Infected = Confirmed - Fatal - Recovered.
         """
         all_df = self._gis.all()
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=all_df, layers=self._layers, date=self.DATE)
         transformer.susceptible(new=self.S, population=self.N, confirmed=self.C)
         transformer.infected(new=self.CI, confirmed=self.C, fatal=self.F, recovered=self.R)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def inverse_transform(self):
         """Perform inverse transformation, calculating total population and confirmed.
@@ -213,16 +208,12 @@ class DataEngineer(Term):
         Note:
             If the alias of values
         """
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.diff(
             column=Validator(
                 self._var_alias.find(name=column, default=[column]), "column", accept_none=False).sequence(length=1)[0],
             suffix=suffix, freq=freq)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def add(self, columns, new=None, fill_value=0):
         """Calculate element-wise addition with pandas.DataFrame.sum(axis=1), X1 + X2 + X3 +...
@@ -236,13 +227,9 @@ class DataEngineer(Term):
             covsirphy.DataEngineer: self
         """
         col_names = self._var_alias.find(name=columns, default=columns)
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.add(columns=col_names, new=new or "+".join(col_names), fill_value=fill_value)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def mul(self, columns, new=None, fill_value=0):
         """Calculate element-wise multiplication with pandas.DataFrame.product(axis=1), X1 * X2 * X3 *...
@@ -256,13 +243,9 @@ class DataEngineer(Term):
             covsirphy.DataEngineer: self
         """
         col_names = self._var_alias.find(name=columns, default=columns)
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.mul(columns=col_names, new=new or "*".join(col_names), fill_value=fill_value)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def sub(self, minuend, subtrahend, new=None, fill_value=0):
         """Calculate element-wise subtraction with pandas.Series.sub(), minuend - subtrahend.
@@ -276,7 +259,6 @@ class DataEngineer(Term):
         Returns:
             covsirphy.DataEngineer: self
         """
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.sub(
             minuend=Validator(
@@ -284,10 +266,7 @@ class DataEngineer(Term):
             subtrahend=Validator(
                 self._var_alias.find(name=subtrahend, default=[subtrahend]), "subtrahend", accept_none=False).sequence(length=1)[0],
             new=new or f"{minuend}-{subtrahend}", fill_value=fill_value)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def div(self, numerator, denominator, new=None, fill_value=0):
         """Calculate element-wise floating division with pandas.Series.div(), numerator / denominator.
@@ -304,7 +283,6 @@ class DataEngineer(Term):
         Note:
             Positive rate could be calculated with Confirmed / Tested, `.div(numerator="Confirmed", denominator="Tested", new="Positive_rate")`
         """
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.div(
             numerator=Validator(
@@ -312,10 +290,7 @@ class DataEngineer(Term):
             denominator=Validator(
                 self._var_alias.find(name=denominator, default=[denominator]), "denominator", accept_none=False).sequence(length=1)[0],
             new=new or f"{numerator}_per_({denominator.replace(' ', '_')})", fill_value=fill_value)
-        self._gis = GIS(**self._gis_kwargs)
-        self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
-        return self
+        return self._recreate_gis(transformer)
 
     def assign(self, **kwargs):
         """Assign a new column with pandas.DataFrame.assign().
@@ -326,26 +301,35 @@ class DataEngineer(Term):
         Note:
             Refer to documentation of pandas.DataFrame.assign(), https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.assign.html
         """
-        citations = self._gis.citations(variables=None)
         transformer = _DataTransformer(data=self._gis.all(), layers=self._layers, date=self.DATE)
         transformer.assign(**kwargs)
+        return self._recreate_gis(transformer)
+
+    def _recreate_gis(self, transformer):
+        """Recreate GIS instance with transformer.
+
+        Args:
+            transformer (class object which has .all() method): transformer
+        """
+        citations = self._gis.citations(variables=None)
         self._gis = GIS(**self._gis_kwargs)
         self._gis.register(
-            data=transformer.all(), layers=self._layers, date=self.DATE, variables=None, citations=citations, convert_iso3=False)
+            data=transformer.all(), layers=self._layers, date=self.DATE,
+            variables=None, citations=citations, convert_iso3=False)
         return self
 
     def layer(self, geo=None, start_date=None, end_date=None, variables=None):
         """Return the data at the selected layer in the date range.
 
         Args:
-            geo (tuple(list[str] or tuple(str) or str) or str or None): location names to specify the layer or None (the top level)
-            start_date (str or None): start date, like 22Jan2020
-            end_date (str or None): end date, like 01Feb2020
-            variables (list[str] or None): list of variables to add or None (all available columns)
+            geo(tuple(list[str] or tuple(str) or str) or str or None): location names to specify the layer or None (the top level)
+            start_date(str or None): start date, like 22Jan2020
+            end_date(str or None): end date, like 01Feb2020
+            variables(list[str] or None): list of variables to add or None (all available columns)
 
         Raises:
-            TypeError: @geo has un-expected types
-            ValueError: the length of @geo is larger than the length of layers
+            TypeError: @ geo has un - expected types
+            ValueError: the length of @ geo is larger than the length of layers
             NotRegisteredError: No records have been registered at the layer yet
 
         Returns:
@@ -354,36 +338,36 @@ class DataEngineer(Term):
                     reset index
                 Columns
                     - (str): columns defined by covsirphy.GIS(layers)
-                    - Date (pandas.Timestamp): observation dates
-                    - columns defined by @variables
+                    - Date(pandas.Timestamp): observation dates
+                    - columns defined by @ variables
 
         Note:
            Note that records with NAs as country names will be always removed.
 
         Note:
-            Regarding @geo argument, please refer to covsirphy.GIS.layer().
+            Regarding @ geo argument, please refer to covsirphy.GIS.layer().
         """
         v_converted = self._var_alias.find(name=variables, default=variables)
         return self._gis.layer(geo=geo, start_date=start_date, end_date=end_date, variables=v_converted, errors="raise")
 
-    def choropleth(self, geo, variable, on=None, title="Choropleth map", filename="choropleth.jpg", logscale=True, directory="input", natural_earth=None, **kwargs):
+    def choropleth(self, geo, variable, on=None, title="Choropleth map", filename="choropleth.jpg", logscale=True, directory=None, natural_earth=None, **kwargs):
         """Create choropleth map.
 
         Args:
-            geo (tuple(list[str] or tuple(str) or str) or str or None): location names to specify the layer or None (the top level)
-            variable (str): variable name to show
-            on (str or None): the date, like 22Jan2020, or None (the last date of each location)
-            title (str): title of the map
-            filename (str or None): filename to save the figure or None (display)
-            logscale (bool): whether convert the value to log10 scale values or not
-            directory (str): directory to save GeoJSON file of "Natural Earth" GitHub repository
-            natural_earth (str or None): title of GeoJSON file (without extension) of "Natural Earth" GitHub repository or None (automatically determined)
+            geo(tuple(list[str] or tuple(str) or str) or str or None): location names to specify the layer or None (the top level)
+            variable(str): variable name to show
+            on(str or None): the date, like 22Jan2020, or None (the last date of each location)
+            title(str): title of the map
+            filename(str or None): filename to save the figure or None (display)
+            logscale(bool): whether convert the value to log10 scale values or not
+            directory(str or None): directory to save GeoJSON file of "Natural Earth" GitHub repository or None (the directory of GIS class script)
+            natural_earth(str or None): title of GeoJSON file(without extension) of "Natural Earth" GitHub repository or None (automatically determined)
             **kwargs: keyword arguments of the following classes and methods.
                 - matplotlib.pyplot.savefig(), matplotlib.pyplot.legend(), and
                 - pandas.DataFrame.plot()
 
         Note:
-            Regarding @geo argument, please refer to covsirphy.GIS.layer().
+            Regarding @ geo argument, please refer to covsirphy.GIS.layer().
 
         Note:
             GeoJSON files are listed in https://github.com/nvkelso/natural-earth-vector/tree/master/geojson
@@ -396,38 +380,38 @@ class DataEngineer(Term):
         gis.register(data=layer_df, date=self.DATE)
         gis.choropleth(
             variable=variable, filename=filename, title=title, logscale=logscale,
-            geo=geo, on=on, directory=[directory, "natural_earth"], natural_earth=natural_earth, **kwargs)
+            geo=geo, on=on, directory=directory, natural_earth=natural_earth, **kwargs)
 
     def subset(self, geo=None, start_date=None, end_date=None, variables=None, complement=True, **kwargs):
         """Return subset of the location and date range.
 
         Args:
-            geo (tuple(list[str] or tuple(str) or str) or str or None): location names to filter or None (total at the top level)
-            start_date (str or None): start date, like 22Jan2020
-            end_date (str or None): end date, like 01Feb2020
-            variables (list[str] or None): list of variables to add or None (all available columns)
+            geo(tuple(list[str] or tuple(str) or str) or str or None): location names to filter or None (total at the top level)
+            start_date(str or None): start date, like 22Jan2020
+            end_date(str or None): end date, like 01Feb2020
+            variables(list[str] or None): list of variables to add or None (all available columns)
             complement (bool): whether perform data complement or not, True as default
             **Kwargs: keyword arguments for complement and default values
-                recovery_period (int): expected value of recovery period [days], 17
-                interval (int): expected update interval of the number of recovered cases [days], 2
-                max_ignored (int): Max number of recovered cases to be ignored [cases], 100
-                max_ending_unupdated (int): Max number of days to apply full complement, where max recovered cases are not updated [days], 14
-                upper_limit_days (int): maximum number of valid partial recovery periods [days], 90
-                lower_limit_days (int): minimum number of valid partial recovery periods [days], 7
-                upper_percentage (float): fraction of partial recovery periods with value greater than upper_limit_days, 0.5
-                lower_percentage (float): fraction of partial recovery periods with value less than lower_limit_days, 0.5
+                recovery_period(int): expected value of recovery period[days], 17
+                interval(int): expected update interval of the number of recovered cases[days], 2
+                max_ignored(int): Max number of recovered cases to be ignored[cases], 100
+                max_ending_unupdated(int): Max number of days to apply full complement, where max recovered cases are not updated[days], 14
+                upper_limit_days(int): maximum number of valid partial recovery periods[days], 90
+                lower_limit_days(int): minimum number of valid partial recovery periods[days], 7
+                upper_percentage(float): fraction of partial recovery periods with value greater than upper_limit_days, 0.5
+                lower_percentage(float): fraction of partial recovery periods with value less than lower_limit_days, 0.5
 
         Returns:
             tuple(pandas.DataFrame, str, dict):
                 pandas.DataFrame
                     Index
-                        Date (pandas.DataFrame): observation dates
+                        Date(pandas.DataFrame): observation dates
                     Columns
-                        Population (int): total population
-                        Tests (int): column of the number of tests
-                        Confirmed (int): the number of confirmed cases
-                        Fatal (int): the number of fatal cases
-                        Recovered (int): the number of recovered cases
+                        Population(int): total population
+                        Tests(int): column of the number of tests
+                        Confirmed(int): the number of confirmed cases
+                        Fatal(int): the number of fatal cases
+                        Recovered(int): the number of recovered cases
                         the other columns registered
                 str: status code: will be selected from
                     - '' (not complemented)
@@ -444,10 +428,10 @@ class DataEngineer(Term):
                     - Partial_recovered
 
         Note:
-            Regarding @geo argument, please refer to covsirphy.GIS.subset().
+            Regarding @ geo argument, please refer to covsirphy.GIS.subset().
 
         Note:
-            Re-calculation of Susceptible and Infected will be done automatically.
+            Re - calculation of Susceptible and Infected will be done automatically.
         """
         v_converted = self._var_alias.find(name=variables, default=variables)
         subset_df = self._gis.subset(geo=geo, start_date=start_date, end_date=end_date, variables=None, errors="raise")
@@ -476,17 +460,17 @@ class DataEngineer(Term):
         return transformed_df.loc[:, v_converted or transformed_df.columns], status, status_dict
 
     def subset_alias(self, alias=None, update=False, **kwargs):
-        """Set/get/list-up alias name(s) of subset.
+        """Set / get / list - up alias name(s) of subset.
 
         Args:
-            alias (str or None): alias name or None (list-up alias names)
-            update (bool): force updating the alias when @alias is not None
+            alias(str or None): alias name or None (list - up alias names)
+            update(bool): force updating the alias when @ alias is not None
             **kwargs: keyword arguments of covsirphy.DataEngineer().subset()
 
         Returns:
             tuple(pandas.DataFrame, str, dict) or dict[str, tuple(pandas.DataFrame, str, dict)]:
-                - tuple(pandas.DataFrame, str, dict): when @alias is not None, the subset of the alias
-                - dict[str, tuple(pandas.DataFrame, str, dict)]: when @alias is None, dictionary of aliases and subsets
+                - tuple(pandas.DataFrame, str, dict): when @ alias is not None, the subset of the alias
+                - dict[str, tuple(pandas.DataFrame, str, dict)]: when @ alias is None, dictionary of aliases and subsets
 
         Note:
             When the alias name was a new one, subset will be registered with covsirphy.DataEngineer.subset(**kwargs).
@@ -499,22 +483,22 @@ class DataEngineer(Term):
         return self._subset_alias.find(alias)
 
     def variables_alias(self, alias=None, variables=None):
-        """Set/get/list-up alias name(s) of variables.
+        """Set / get / list - up alias name(s) of variables.
 
         Args:
-            alias (str or None): alias name or None (list-up alias names)
-            variables (list[str]): variables to register with the alias
+            alias(str or None): alias name or None (list - up alias names)
+            variables(list[str]): variables to register with the alias
 
         Raises:
-            NotIncludedError: the alias is not None and un-registered
+            NotIncludedError: the alias is not None and un - registered
 
         Returns:
             list[str] or dict[str, list[str]]:
-                - list[str]: when @alias is not None, the variables of the alias
-                - dict[str, list[str]]: when @alias is None, dictionary of aliases and variables
+                - list[str]: when @ alias is not None, the variables of the alias
+                - dict[str, list[str]]: when @ alias is None, dictionary of aliases and variables
 
         Note:
-            When @variables is not None, alias will be registered/updated.
+            When @ variables is not None, alias will be registered / updated.
 
         Note:
             Some aliases are preset. We can check them with covsirphy.DataEngineer().variables_alias().
@@ -532,17 +516,17 @@ class DataEngineer(Term):
         """Calculate mode value of recovery period of the data.
 
         Args:
-            data (pandas.DataFrame): data for calculation
+            data(pandas.DataFrame): data for calculation
                 Index
-                    Date (pandas.Timestamp): observation dates
+                    Date(pandas.Timestamp): observation dates
                 Columns
-                    Confirmed (int): the number of confirmed cases, optional
-                    Fatal (int): the number of fatal cases, optional
-                    Recovered (int): the number of recovered cases, optional
+                    Confirmed(int): the number of confirmed cases, optional
+                    Fatal(int): the number of fatal cases, optional
+                    Recovered(int): the number of recovered cases, optional
                     the other columns will be ignored
 
         Returns:
-            int: mode value of recovery period [days]
+            int: mode value of recovery period[days]
         """
         df = Validator(data, "data").dataframe(time_index=True, columns=[cls.C, cls.F, cls.R], empty_ok=False)
         df = df.resample("D").sum()
