@@ -1,16 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from pathlib import Path
 import geopandas as gpd
 from matplotlib import pyplot as plt
 import pandas as pd
 from covsirphy.util.error import NotRegisteredError, SubsetNotFoundError
 from covsirphy.util.validator import Validator
 from covsirphy.util.term import Term
-from covsirphy.gis.subset import _SubsetManager
-from covsirphy.gis.layer import _LayerAdjuster
-from covsirphy.gis.geometry import _Geometry
-from covsirphy.gis.choropleth import _ChoroplethMap
+from covsirphy.gis._subset import _SubsetManager
+from covsirphy.gis._layer import _LayerAdjuster
+from covsirphy.gis._geometry import _Geometry
+from covsirphy.gis._choropleth import _ChoroplethMap
 
 
 class GIS(Term):
@@ -157,14 +158,14 @@ class GIS(Term):
         """
         # Get all data
         if self._un_registered and errors == "raise":
-            raise NotRegisteredError("GIS.register()", details="No records have been registered yet.")
+            raise NotRegisteredError("GIS.register()", details="No records have been registered yet")
         data = self._adjuster.all(variables=variables)
         # Filter with geo
         geo_converted = self._parse_geo(geo=geo, data=data)
         manager = _SubsetManager(layers=self._layers)
         df = manager.layer(data=data, geo=geo_converted)
         if df.empty and errors == "raise":
-            raise NotRegisteredError("GIS.register()", details="No records have been registered at the layer yet.")
+            raise NotRegisteredError("GIS.register()", details="No records have been registered at the layer yet")
         # Filter with date
         series = df[self._date].copy()
         start = Validator(start_date).date(default=series.min())
@@ -172,7 +173,7 @@ class GIS(Term):
         df = df.loc[(df[self._date] >= start) & (df[self._date] <= end)]
         if df.empty and errors == "raise":
             raise NotRegisteredError(
-                "GIS.register()", details=f"No records have been registered at the layer yet from {start_date} to {end_date}.")
+                "GIS.register()", details=f"No records have been registered at the layer yet from {start_date} to {end_date}")
         # Get representative records for dates
         df = df.groupby([*self._layers, self._date], dropna=True).first()
         return df.reset_index().convert_dtypes()
@@ -184,7 +185,7 @@ class GIS(Term):
             geo (tuple(list[str] or tuple(str) or str) or str or None): location names to specify the layer or None (the top level)
             on (str or None): the date, like 22Jan2020, or None (the last date of each location)
             variables (list[str] or None): list of variables to add or None (all available columns)
-            directory (list[str] or tuple(str) or str): top directory name(s) to save GeoJSON files or None (["input", "natural_earth"])
+            directory (list[str] or tuple(str) or str): top directory name(s) to save GeoJSON files or None (directory of this this script)
             natural_earth (str or None): title of GeoJSON file (without extension) of "Natural Earth" GitHub repository or None (automatically determined)
 
         Raises:
@@ -217,7 +218,7 @@ class GIS(Term):
             df = df.loc[df[self._date] == Validator(on).date()]
         focused_layer = [layer for layer in self._layers if df[layer][df[layer] != self.NA].nunique() > 0][-1]
         geometry = _Geometry(
-            data=df, layer=focused_layer, directory=directory or ["input", "natural_earth"], verbose=self._verbose)
+            data=df, layer=focused_layer, directory=directory or Path(__file__).with_name("Natural_Earth"), verbose=self._verbose)
         iso3 = None if focused_layer == self._country else self._to_iso3(list(df[self._country].unique())[0])
         return geometry.to_geopandas(iso3=iso3, natural_earth=natural_earth).drop(set(self._layers) - {focused_layer}, axis=1)
 
@@ -340,7 +341,7 @@ class GIS(Term):
                     - (str): column defined by @country (of covsirphy.GIS) if @country is not None
 
         Returns:
-            geo (tuple(list[str] or tuple(str) or str) or str or None): parsed location names
+            geo (tuple(list[str] or tuple(str) or str or None) or str or None): parsed location names
         """
         if geo is None:
             return geo
